## pyRAD-3.063 version
./pyrad/pyRAD.py -d analysis_dtests/input.1.fs

mkdir -p analysis_dtests

import numpy as np
import scipy.stats
import itertools
import gzip
from collections import OrderedDict, Counter

## create D-statistic template input file
! ~/pyrad_v.2.13/pyRAD -D > analysis_dtests/input.template
! cat analysis_dtests/input.template

## Python function for editing input files
## Takes a list of sample names for each taxon
## and writes iterations to file, uses template file

 
def makeD4(P1,P2,P3,O,poly,infile):
    ! head -n 8 $infile > temp
    ! mv temp $infile
    tests = []
    if not poly:
        for o in O:
            for p3 in P3:
                for p2 in P2:
                    for p1 in P1:
                        if p1 != p2:
                            if set((p1,p2,p3,o)) not in tests:
                                tests.append(set((p1,p2,p3,o)))
                                test = '%s\t%s\t%s\t%s\t##' % (p1,p2,p3,o)
                                ## appends tests to input file
                                ! echo '$test' >> analysis_dtests/input.template
    else:
        for p3 in P3:
            for p2 in P2:
                for p1 in P1:
                    if p1 != p2:
                        if set((p1,p2,p3)) not in tests:
                            tests.append(set((p1,p2,p3)))
                            test = '[%s]\t[%s]\t[%s]\t[%s]\t##' % (p1,p2,p3,",".join(O))
                            ## appends tests to input file
                            ! echo '$test' >> analysis_dtests/input.template


def makeD4(P1,P2,P3,O,num,name,template):
	stringout = ""
	tests = []
	i = 1
	for p1 in P1:
		for p2 in P2:
			for p3 in P3:
				if p1 != p2:
					if set([p1,p2,p3]) not in tests:
						stringout += "[%s]\t[%s]\t[%s]\t[%s]\t## test %i.%i\n" %(p1,p2,p3,",".join(O),num,i)
						i += 1
						tests.append(set([p1,p2,p3]))
	! head -n 8 $template > "analysis_dtests/input."$num"."$name".txt"
	! echo "$stringout" >> "analysis_dtests/input."$num"."$name".txt"
	ostring = "/## output file/c\\analysis_dtests/out."+str(num)+"."+str(name)+"  ## output file "
	! sed -i "$ostring" "analysis_dtests/input."$num"."$name".txt"

	
## 5 groups
A=['07_F261_TR_A','11_G386_KZ_A','11_G595_KZ_A','11_G663_KZ_A','14_G632_GR_A']
B=['11_H582_IT_B','08_R519_IT_B','11_H748_IT_B','11_I076_IT_B','11_H995_IT_B']
C=['11_I689_FR_C','11_J146_CH_C','12_O852_FR_C','14_H910_FR_C','15_L572_IT_C']
D=['06_G603_MO_D','11_F172_MO_D','11_F879_MO_D','12_L827_MO_D','12_N510_TN_D']
E=['08_J709_SP_E','08_J735_SP_E','08_J993_SP_E','08_P370_SP_E','14_V970_SP_E']
out=['14_I563_IT_out','15_A753_PT_out'] 

makeD4(A,A,B,out,1,"analysis_dtests/input.template") ##1
makeD4(A,A,C,out,1,"analysis_dtests/input.template") ##2
makeD4(A,A,D,out,1,"analysis_dtests/input.template") ##3
makeD4(A,A,E,out,1,"analysis_dtests/input.template") ##4
makeD4(B,B,A,out,1,"analysis_dtests/input.template") ##5
makeD4(B,B,C,out,1,"analysis_dtests/input.template") ##6
makeD4(B,B,D,out,1,"analysis_dtests/input.template") ##7
makeD4(B,B,E,out,1,"analysis_dtests/input.template") ##8
makeD4(C,C,A,out,1,"analysis_dtests/input.template") ##9
makeD4(C,C,B,out,1,"analysis_dtests/input.template") ##10
makeD4(C,C,D,out,1,"analysis_dtests/input.template") ##11
makeD4(C,C,E,out,1,"analysis_dtests/input.template") ##12
makeD4(D,D,A,out,1,"analysis_dtests/input.template") ##13
makeD4(D,D,B,out,1,"analysis_dtests/input.template") ##14
makeD4(D,D,C,out,1,"analysis_dtests/input.template") ##15
makeD4(D,D,E,out,1,"analysis_dtests/input.template") ##16
makeD4(E,E,A,out,1,"analysis_dtests/input.template") ##17
makeD4(E,E,B,out,1,"analysis_dtests/input.template") ##18
makeD4(E,E,C,out,1,"analysis_dtests/input.template") ##19
makeD4(E,E,D,out,1,"analysis_dtests/input.template") ##20


## run tests 1-10
for test in range(1,10):
	t = glob.glob("analysis_dtests/input."+str(test)+".*")[0]
	stderr = ! ~/Dropbox/pyrad-git/pyRAD -d $t


	## strips junk from Dtest results files to make names match in G dict'
def nname(x):
	x = x.replace("]","").replace("[","").strip()
	return x

## returns number of significant tests at some alpha level of significance
## after sequential bonferoni correction.
from itertools import groupby
import scipy.stats
def correct(Zs,alpha):
	""" performs Holm-bonferroni correction on a list of Z-scores """
	PS = []
	Zs = list(Zs)
	pone = map(scipy.stats.distributions.norm.sf, Zs)
	pvals = map(lambda x: x*2, pone) ## two-sided test
	pvals_idxs = zip(pvals, xrange(len(pvals)))
	pvals_idxs.sort()
	lp = len(pvals)
	for pval, idxs in groupby(pvals_idxs, lambda x: x[0]):
		idxs = list(idxs)
		for p, i in idxs:
			if p < alpha/lp:
				PS.append(i)
		lp -= len(idxs)
	return [len(PS),len(Zs),min(Zs),max(Zs)]

## reads in Z scores from the pyRAD output and return summary
def getZs(infile,Ntest,exclude=[]):
	""" returns the Z-scores from a PyRAD Dtest output file """
	lines = open(infile,'r').readlines()
	header, data = lines[0:1],lines[2:]
	dat1 = [i.strip().split("\t") for i in data]
	for name in exclude:
		dat1 = [line for line in dat1 if name not in "".join(line)]
	D = np.array([i for i in dat1])
	Za = []
	Zb = []
	Zab = []
	taxorder = []
	L = []
	if Ntest == 5:
		for line in dat1:
			p1,p2,p31,p32,o = line[0:5]
			Zab.append(line[8])
			Za.append(line[9])
			Zb.append(line[10])
			L.append(line[17])
			taxorder.append([p1,p2,p31,p32,o])
		RR = [[map(float,Zab),map(float,Za),map(float,Zb)],min(map(int,L)),max(map(int,L)),taxorder]
	if Ntest == 4:
		taxorder = zip([D[0],D[1],D[2]])
		Z,L = D[:,6],D[:,9]
		RR = [map(float,Z), min(map(int,L)), max(map(int,L)),taxorder]
	return RR

## for printing results of bonferoni test
def retHB(infile, Ntest, alpha, exclude=[], header=False, ptax=False):
	""" combines the previous two functions into one, and returns the output in pretty format """
	if Ntest == 4:
		if header == True:
			print "\t".join(["range_Z", "N tests/Sig", "range_N_loci"])
		Zs,minL,maxL,taxorder = getZs(infile, Ntest, exclude)
		s,t,minz,maxz = correct(Zs, alpha)
		return [minz,maxz,s,t,minL,maxL]
	elif Ntest == 5:
		if header == True:
			print "\t".join(["range_Z12", "Sig","range_Z1", "Sig","range_Z2", "Sig","range_N_loci"])
		ZS,minL,maxL,taxorder = getZs(infile, Ntest, exclude)
		s12,t12,minz12,maxz12 = correct(ZS[0], alpha)
		s1,t1,minz1,maxz1 = correct(ZS[1], alpha)
		s2,t2,minz2,maxz2 = correct(ZS[2], alpha)
		return [minz12,maxz12,s12,t12,minz1,maxz1,s1,t1,minz2,maxz2,s2,t2,minL,maxL]

## Summarized results
import glob
infiles = glob.glob("analysis_dtests/*.D4.txt")
infiles.sort(key=lambda x: int(x.split('.')[1]))
D = {}
for ff in infiles:
	rr = retHB(ff,4,0.01,[],0)
	ss = "\t".join(map(str,[ff.split("/")[-1]]))
	D[ff.split("/")[-1][4:-7]] = tuple(rr)
	print "%s \t(%.1f - %.1f)\t%s/%s\t(%s - %s)" % (tuple([ff.split("/")[-1]])+ tuple(rr))

## Introgression from the outs
import pandas as pd
print pd.read_table("analysis_dtests/dtest_t2.template.txt")



















